#------------------------------------------------------------------------------
# Copyright (c) 2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
""" Test the Dialog widget.

"""
from utils import compile_source, wait_for_window_displayed

SOURCE = """
from enaml.widgets.api import Container, Dialog, Label, MainWindow, PushButton

enamldef TestDialog(Dialog): dialog:

    title = "Test Dialog"

    alias cancel_button:cancel_button
    alias ok_button:ok_button

    Container:
        PushButton: ok_button:
            text = "Ok"
            clicked::
                dialog.accept()
        PushButton: cancel_button:
            text = "Cancel"
            clicked::
                dialog.reject()

enamldef Main(MainWindow): window:

    title = "Main Test Window"

    alias async_button:async_button

    attr starting = 0
    attr started  = 0
    attr finished = 0
    attr accepted = 0
    attr rejected = 0

    func do_starting():
        window.starting += 1

    func do_started():
        window.started += 1

    func do_finished():
        window.finished += 1

    func do_accepted():
        window.accepted += 1

    func do_rejected():
        window.rejected += 1

    Container:

        Label:
            text = f"Starting: {window.starting}"

        Label:
            text = f"Started:  {window.started}"

        Label:
            text = f"Finished: {window.finished}"

        Label:
            text = f"Accepted: {window.accepted}"

        Label:
            text = f"Rejected: {window.rejected}"

        PushButton: async_button:
            text = "Launch Dialog Asynchronously"
            clicked::
                window.do_starting()
                dialog = TestDialog(self)
                dialog.finished.bind(lambda e: window.do_finished())
                dialog.accepted.bind(lambda e: window.do_accepted())
                dialog.rejected.bind(lambda e: window.do_rejected())
                dialog.open()
                window.do_started()
"""

def test_dialog_asynchronous(enaml_qtbot):

    from enaml.qt import QtCore

    window = compile_source(SOURCE, "Main")()
    window.show()
    wait_for_window_displayed(enaml_qtbot, window)

    assert len(window.windows) == 1
    assert window.starting == 0
    assert window.started  == 0
    assert window.finished == 0
    assert window.accepted == 0
    assert window.rejected == 0

    # click the button to launch the dialog.
    enaml_qtbot.mouseClick(window.async_button.proxy.widget, QtCore.Qt.LeftButton)

    assert len(window.windows) == 2
    assert window.starting == 1
    assert window.started  == 1
    assert window.finished == 0
    assert window.accepted == 0
    assert window.rejected == 0

    # click the Ok button in the dialog.
    enaml_qtbot.mouseClick(next(otherWindow for otherWindow in window.windows if otherWindow.title == 'Test Dialog').ok_button.proxy.widget, QtCore.Qt.LeftButton)
    enaml_qtbot.waitUntil(lambda: len(window.windows) == 1)

    # FIXME it appears as if dialogs do not properly unregister themselves from the window set when closed and out of scope.
    assert len(window.windows) == 1
    assert window.starting == 1
    assert window.started  == 1
    assert window.finished == 1
    assert window.accepted == 1
    assert window.rejected == 0

    # click the button to launch the dialog.
    enaml_qtbot.mouseClick(window.async_button.proxy.widget, QtCore.Qt.LeftButton)

    assert len(window.windows) == 2
    assert window.starting == 2
    assert window.started  == 2
    assert window.finished == 1
    assert window.accepted == 1
    assert window.rejected == 0

    # click the Cancel button in the dialog.
    enaml_qtbot.mouseClick(next(otherWindow for otherWindow in window.windows if otherWindow.title == 'Test Dialog').cancel_button.proxy.widget, QtCore.Qt.LeftButton)
    enaml_qtbot.waitUntil(lambda: len(window.windows) == 1)

    assert len(window.windows) == 1
    assert window.starting == 2
    assert window.started  == 2
    assert window.finished == 2
    assert window.accepted == 1
    assert window.rejected == 1



#------------------------------------------------------------------------------
# Copyright (c) 2013-2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
from abc import ABCMeta
from collections.abc import Iterable, Iterator

from atom.api import Atom, Int, Coerced, List, Typed, Value
from atom.datastructures.api import sortedmap

from .compiler_nodes import new_scope
from .declarative import d_
from .pattern import Pattern


def coerce_iterable(iterable):
    """ Coerce iterators to a tuple or return the iterable as is.

    """
    if isinstance(iterable, Iterator):
        return tuple(iterable)
    elif isinstance(iterable, Iterable):
        return iterable
    raise TypeError("%s object is not iterable" % type(iterable))


class LooperIterableMeta(ABCMeta):
    """ Metaclass which checks if an instance is Iterable but not an Iterator.

    """
    def __instancecheck__(self, instance):
        if isinstance(instance, Iterator):
            return False
        return isinstance(instance, Iterable)


class LooperIterable(Iterable, metaclass=LooperIterableMeta):
    """ An Iterable that is not an Iterator

    """


class Iteration(Atom):
    """ A container to hold data for items in the Looper.

    """
    #: Index within the Looper's iterable
    index = Int()

    #: Item from the Looper's iterable
    item = Value()

    #: Nodes generated by the Looper
    nodes = List()


class Looper(Pattern):
    """ A pattern object that repeats its children over an iterable.

    The children of a `Looper` are used as a template when creating new
    objects for each item in the given `iterable`. Each iteration of the
    loop will be given an independent scope which is the union of the
    outer scope and any identifiers created during the iteration. This
    scope will also contain a `loop` variable which has `item` and `index`
    members to access the index and value of the iterable, respectively.

    All items created by the looper will be added as children of the
    parent of the `Looper`. The `Looper` keeps ownership of all items
    it creates. When the iterable for the looper is changed, the looper
    will only create and destroy children for the items in the iterable
    which have changed. When an item in the iterable is moved the
    `loop.index` will be updated to reflect the new index.

    The Looper works under the assumption that the values stored in the
    iterable are unique.

    The `loop_item` and `loop_index` scope variables are depreciated in favor
    of `loop.item` and `loop.index` respectively. This is because the old
    `loop_index` variable may become invalid when items are moved.

    """
    #: The iterable to use when creating the items for the looper.
    #: The items in the iterable must be unique. This allows the
    #: Looper to optimize the creation and destruction of widgets.
    #: If the iterable is an Iterator it is first coerced to a tuple.
    iterable = d_(Coerced(LooperIterable, coercer=coerce_iterable))

    #: The list of items created by the conditional. Each item in the
    #: list represents one iteration of the loop and is a list of the
    #: items generated during that iteration. This list should not be
    #: manipulated directly by user code.
    items = List()

    #: Private data storage which maps the user iterable data to the
    #: list of items created for that iteration. This allows the looper
    #: to only create and destroy the items which have changed.
    _iter_data = Typed(sortedmap, ())

    #--------------------------------------------------------------------------
    # Lifetime API
    #--------------------------------------------------------------------------
    def destroy(self):
        """ A reimplemented destructor.

        The looper will release the owned items on destruction.

        """
        super(Looper, self).destroy()
        del self.iterable
        del self.items
        del self._iter_data

    #--------------------------------------------------------------------------
    # Observers
    #--------------------------------------------------------------------------
    def _observe_iterable(self, change):
        """ A private observer for the `iterable` attribute.

        If the iterable changes while the looper is active, the loop
        items will be refreshed.

        """
        if change['type'] == 'update' and self.is_initialized:
            self.refresh_items()

    #--------------------------------------------------------------------------
    # Pattern API
    #--------------------------------------------------------------------------
    def pattern_items(self):
        """ Get a list of items created by the pattern.

        """
        return sum(self.items, [])

    def refresh_items(self):
        """ Refresh the items of the pattern.

        This method destroys the old items and creates and initializes
        the new items.

        """
        old_items = self.items[:]
        old_iter_data = self._iter_data
        iterable = self.iterable
        pattern_nodes = self.pattern_nodes
        new_iter_data = sortedmap()
        new_items = []

        if iterable is not None and len(pattern_nodes) > 0:
            for loop_index, loop_item in enumerate(iterable):
                iter_data = old_iter_data.get(loop_item)
                if iter_data is not None:
                    new_iter_data[loop_item] = iter_data
                    iteration = iter_data.nodes
                    new_items.append(iteration)
                    old_items.remove(iteration)
                    iter_data.index = loop_index
                    continue
                iter_data = Iteration(index=loop_index, item=loop_item)
                iteration = iter_data.nodes
                new_iter_data[loop_item] = iter_data
                new_items.append(iteration)
                for nodes, key, f_locals in pattern_nodes:
                    with new_scope(key, f_locals) as f_locals:
                        # Retain for compatibility reasons
                        f_locals['loop_index'] = loop_index
                        f_locals['loop_item'] = loop_item
                        f_locals['loop'] = iter_data
                        for node in nodes:
                            child = node(None)
                            if isinstance(child, list):
                                iteration.extend(child)
                            else:
                                iteration.append(child)

        for iteration in old_items:
            for old in iteration:
                if not old.is_destroyed:
                    old.destroy()

        if len(new_items) > 0:
            expanded = []
            recursive_expand(sum(new_items, []), expanded)
            self.parent.insert_children(self, expanded)

        self.items = new_items
        self._iter_data = new_iter_data


def recursive_expand(items, expanded):
    """ Recursively expand the list of items created by the looper.

    This allows the final list to be inserted into the parent and
    maintain the proper ordering of children.

    Parameters
    ----------
    items : list
        The list of items to expand. This should be composed of
        Pattern and other Object instances.

    expanded : list
        The output list. This list will be modified in-place.

    """
    for item in items:
        if isinstance(item, Pattern):
            recursive_expand(item.pattern_items(), expanded)
        expanded.append(item)



#------------------------------------------------------------------------------
# Copyright (c) 2013-2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
""" A simple example plugin application.

This example serves to demonstrate the concepts described the accompanying
developer crash source document.

"""
import enaml
from enaml.workbench.ui.api import UIWorkbench

import sys, os
sys.path.append(os.path.dirname(os.path.abspath(__file__)))

def main():
    with enaml.imports():
        from sample_plugin import SampleManifest

    workbench = UIWorkbench()
    workbench.register(SampleManifest())
    workbench.run()


if __name__ == '__main__':
    main()



# ------------------------------------------------------------------------------
# Copyright (c) 2018-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ------------------------------------------------------------------------------
import sys
import ast
import pytest

from enaml.core.parser import parse

from .test_parser import validate_ast

TEST_SOURCE = {
    "f-string single value": r"""
    a = f'test {a:d}'
    """,
    "f-string multiple values": r"""
    a = f'test {a:g}, {b}'
    """,
    "f-string split 1": r"""
    a = (f'{r}' '{t:s}')
    """,
    "f-string split 2": r"""
    a = ('{r}' f'{t:s}')
    """,
    "f-string raw string": r"""
    a = rf'{a}\n'
    """,
    "f-string raw string 2": r"""
    a = fr'{a}\n'
    """,
}
for k, v in list(TEST_SOURCE.items()):
    TEST_SOURCE[k.replace("f-", "F-")] = v.replace("f'", "F'")
    if "rf'" in v:
        TEST_SOURCE[k.replace("raw", "RAW")] = v.replace("rf'", "Rf'")
        TEST_SOURCE[k.replace("f-", "F-").replace("raw", "RAW")] = v.replace(
            "rf'", "RF'"
        )
    if "fr'" in v:
        TEST_SOURCE[k.replace("raw", "RAW")] = v.replace("fr'", "fR'")
        TEST_SOURCE[k.replace("f-", "F-").replace("raw", "RAW")] = v.replace(
            "fr'", "FR'"
        )


@pytest.mark.skipif(sys.version_info < (3, 6), reason="Requires Python 3.6")
@pytest.mark.parametrize("desc", TEST_SOURCE.keys())
def test_f_strings(desc):
    """Test that we produce valid ast for f-strings."""
    src = TEST_SOURCE[desc].strip()
    print(src)
    # Ensure it's valid
    py_ast = ast.parse(src)
    enaml_ast = parse(src).body[0].ast
    validate_ast(py_ast.body[0], enaml_ast.body[0], True)


@pytest.mark.skipif(
    (3, 12) < sys.version_info or sys.version_info < (3, 6),
    reason="Requires Python 3.6",
)
@pytest.mark.parametrize("source", [("f'{\\}'"), ("('d'\nf'{\\}')")])
def test_reporting_errors_f_strings(source):
    """Test that we properly report error on f-string."""
    with pytest.raises(SyntaxError) as e:
        parse(source)

    assert "backslash" in e.value.args[0]



#------------------------------------------------------------------------------
# Copyright (c) 2018-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
import os
import sys
import ast
import enaml
import pytest
import traceback
from textwrap import dedent

from enaml.compat import PY310, PY311


def validate_ast(py_node, enaml_node, dump_ast=False, offset=0):
    """Validate each node of an ast against another ast.

    Typically used to compare an AST generated by the Python paser and one
    generated by the enaml parser.

    """
    if dump_ast:
        print('Python node:\n', ast.dump(py_node))
        print('Enaml node:\n', ast.dump(enaml_node))
    assert type(py_node) == type(enaml_node)
    if isinstance(py_node, ast.AST):
        for name, field in ast.iter_fields(py_node):
            if name == 'ctx':
                assert type(field) == type(getattr(enaml_node, name))
            else:
                field2 = getattr(enaml_node, name, None)
                print('    '*offset, 'Validating:', name)
                validate_ast(field, field2, offset=offset+1)
    elif isinstance(py_node, list):
        if len(py_node) != len(enaml_node):
            return False
        for i, n1 in enumerate(py_node):
            print('    '*offset, 'Validating', i+1, 'th element')
            validate_ast(n1, enaml_node[i], offset=offset+1)
    else:
        assert py_node == enaml_node


def test_syntax_error_traceback_correct_path(tmpdir):
    """ Test that a syntax error retains the path to the file

    """
    test_module_path = os.path.join(tmpdir.strpath, 'view.enaml')

    with open(os.path.join(tmpdir.strpath, 'test_main.enaml'), 'w') as f:
        f.write(dedent("""
        from enaml.widgets.api import Window, Container, Label
        from view import CustomView

        enamldef MyWindow(Window): main:
            CustomView:
                pass

        """))

    with open(test_module_path, 'w') as f:
        f.write(dedent("""
        from enaml.widgets.api import Container, Label

        enamldef CustomLabel(Container):
            Label # : missing intentionally
                text = "Hello world"

        """))

    try:
        sys.path.append(tmpdir.strpath)
        with enaml.imports():
            from test_main import MyWindow
        assert False, "Should raise a syntax error"
    except Exception as e:
        tb = traceback.format_exc()
        print(tb)
        lines = tb.strip().split("\n")
        line = '\n'.join(lines[-4:])
        expected = 'File "{}", line 5'.format(test_module_path)
        assert expected in line
    finally:
        sys.path.remove(tmpdir.strpath)


def test_syntax_error_traceback_show_line(tmpdir):
    """ Test that a syntax error retains the path to the file

    """
    test_module_path = os.path.join(tmpdir.strpath, 'test_syntax.enaml')

    with open(test_module_path, 'w') as f:
        f.write(dedent("""
        from enaml.widgets.api import Container, Label

        enamldef CustomLabel(Container):
            Label # : missing intentionally
                text = "Hello world"
        """))

    try:
        sys.path.append(tmpdir.strpath)
        with enaml.imports():
            from test_syntax import CustomLabel
        assert False, "Should raise a syntax error"
    except Exception as e:
        tb = traceback.format_exc()
        print(tb)
        lines = tb.strip().split("\n")
        line = '\n'.join(lines[-4:])

        expected = 'Label # : missing intentionally'
        assert expected in line
    finally:
        sys.path.remove(tmpdir.strpath)


INDENTATION_TESTS = {
     "enamldef-block": (
        """
        from enaml.widgets.api import Window, Container, Label

        enamldef MainWindow(Window):
        attr x = 1
        """,
        "attr x = 1",
    ),
    "childdef-block": (
        """
        from enaml.widgets.api import Window, Container, Label

        enamldef MainWindow(Window):
            Container:
            Label: # no indent
                text = "Hello world"
        """,
        "Label: # no indent",
    ),
    "childdef-indent-mismatch": (
        """
        from enaml.widgets.api import Window, Container, Label

        enamldef MainWindow(Window):
            Container:
                Label:
                    text = "Hello world"
                 Label: # indent mismatch
                    text = "Hello world"
        """,
        "Label: # indent mismatch",
    ),
    "childdef-attr": (
        """
        from enaml.widgets.api import Window, Container, Label

        enamldef MainWindow(Window):
            Container:
                Label:
                text = 'Hello world'
        """,
        "text = 'Hello world'"
    ),
    "if-block": (
        """
        from enaml.widgets.api import Window

        enamldef MainWindow(Window):
            func go():
                if True:
                x = 1
                else:
                    x = 0
        """,
        "x = 1",
    ),
    "for-block": (
        """
        from enaml.widgets.api import Window

        enamldef MainWindow(Window):
            func go():
                x = 0
                for i in range(4):
                x += 1
        """,
        "x += 1"
    ),
    "try-block": (
        """
        from enaml.widgets.api import Window
        enamldef MainWindow(Window):
            func go():
                try:
                x = 1/0
                except Exception as e:
                    print(e)
        """,
        "x = 1/0"
    ),
    "except-block": (
        """
        from enaml.widgets.api import Window

        enamldef MainWindow(Window):
            func go():
                try:
                    x = 0
                except Exception as e:
                print(e)
        """,
        "print(e)"
    ),
    "finally-block": (
        """
        from enaml.widgets.api import Window

        enamldef MainWindow(Window):
            func go():
                try:
                    x = 0
                finally:
                x = 2
                return 3
        """,
        "x = 2"
    ),
    "class": (
        """
        from enaml.widgets.api import Window, Container, Label

        class Foo:
            x = 1
           def add():
               self.x += 1
        """,
        "def add()",
    ),
}

@pytest.mark.parametrize("label", INDENTATION_TESTS.keys())
def test_indent_error_traceback_show_line(tmpdir, label):
    """ Test that a syntax error retains the path to the file

    """
    test_module_path = os.path.join(tmpdir.strpath, f'test_indent_{label}.enaml')
    source, expected= INDENTATION_TESTS[label]
    with open(test_module_path, 'w') as f:
        f.write(dedent(source.lstrip("\n")))
    try:
        sys.path.append(tmpdir.strpath)
        with enaml.imports():
            __import__(f"test_indent_{label}")
        assert False, "Should raise a identation error"
    except IndentationError as e:
        tb = traceback.format_exc()
        print(tb)
        lines = tb.strip().split("\n")
        line = '\n'.join(lines[-4:])
        assert expected in line
    finally:
        sys.path.remove(tmpdir.strpath)



# ------------------------------------------------------------------------------
# Copyright (c) 2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ------------------------------------------------------------------------------
"""Test importing modules from the stdlib."""


def test_dialog_buttons():
    import enaml
    with enaml.imports():
        import enaml.stdlib.dialog_buttons


def test_dock_area_styles():
    import enaml
    with enaml.imports():
        import enaml.stdlib.dock_area_styles


def test_fields():
    import enaml
    with enaml.imports():
        import enaml.stdlib.fields


def test_mapped_view():
    import enaml
    with enaml.imports():
        import enaml.stdlib.mapped_view


def test_task_dialog():
    import enaml
    with enaml.imports():
        import enaml.stdlib.task_dialog


def test_message_box():
    import enaml
    with enaml.imports():
        import enaml.stdlib.message_box



#------------------------------------------------------------------------------
# Copyright (c) 2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
"""Test the re-ordering of the Notebook tabs."""
import random

from utils import compile_source, wait_for_window_displayed


SOURCE = """
from enaml.widgets.api import MainWindow, Container, PushButton, Notebook, Label, Page
from enaml.core.api import Looper

enamldef Main(MainWindow): mw:

    attr tabs = ["Tab 1", "Tab 2", "Tab 3", "Tab 4"]
    alias notebook: nb

    Container:
        Looper:
            iterable << mw.tabs
            Label:
                text << loop.item

        Notebook: nb:
            Looper:
                iterable << mw.tabs
                Page:
                    closable = False
                    title << loop.item
                    name << loop.item
                    Container:
                        Label:
                            text << loop.item
"""


def get_qt_tab_order(qt_notebook):
    # Assumes QTabWidget-like API
    return [qt_notebook.tabText(i) for i in range(qt_notebook.count())]

def test_tab_order_matches_declaration_after_reorder(enaml_qtbot, enaml_sleep):

    win = compile_source(SOURCE, "Main")()
    win.show()
    wait_for_window_displayed(enaml_qtbot, win)

    assert win.tabs == get_qt_tab_order(win.notebook.proxy.widget)

    new_order = random.sample(win.tabs, len(win.tabs))
    win.tabs = new_order
    enaml_qtbot.wait(enaml_sleep)

    assert win.tabs == get_qt_tab_order(win.notebook.proxy.widget)



#------------------------------------------------------------------------------
# Copyright (c) 2017-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------

TANGO_THEME = {
    "python": {
        "comment": {
            "color": "#8f5902",
            "font-style": "italic"
        },
        "single_quoted_string": {
            "color": "#4e9a06"
        },
        "class_name": {
            "color": "#000000"
        },
        "function_method_name": {
            "color": "#000000"
        },
        "operator": {
            "color": "#ce5c00",
            "font-weight": "bold"
        },
        "double_quoted_string": {
            "color": "#4e9a06"
        },
        "triple_double_quoted_string": {
            "color": "#4e9a06"
        },
        "decorator": {
            "color": "#5c35cc",
            "font-weight": "bold"
        },
        "comment_block": {
            "color": "#8f5902",
            "font-style": "italic"
        },
        "keyword": {
            "color": "#204a87",
            "font-weight": "bold"
        },
        "triple_single_quoted_string": {
            "color": "#4e9a06"
        },
        "unclosed_string": {
            "color": "#a40000",
            "border": "1px solid #ef2929"
        },
        "highlighted_identifier": {
            "paper": "#ffffcc"
        }
    },
    "settings": {
        "caret": "#000000",
        "color": "#000000",
        "paper": "#f8f8f8",
        "name": "tango"
    }
}

TANGO_THEME['enaml'] = TANGO_THEME['python']



#------------------------------------------------------------------------------
# Copyright (c) 2013-2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-
#
# Enaml documentation build configuration file, created by
# sphinx-quickstart on Mon Aug 22 09:16:47 2011.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os
from packaging.version import parse

from enaml.version import __version__

from enaml.core.import_hooks import EnamlImporter
EnamlImporter.install()

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('.'))
sys.path.insert(0, os.path.abspath('./_sphinxext'))
sys.path.insert(0, os.path.abspath('../'))

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
needs_sphinx = '1.4'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.napoleon',
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.imgmath',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.graphviz',
    'sphinx.ext.inheritance_diagram',
    'sphinx.ext.autosummary',
    ]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The main toctree document.
main_doc = 'index'

# General information about the project.
project = 'Enaml'
copyright = '2013-2025, Nucleic Development Team'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
_v = parse(__version__)
version = '{}.{}'.format(_v.major, _v.minor)
# The full version, including alpha/beta/rc tags.
release = __version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_templates', '_sphinxext']

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'trac'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

todo_include_todos = True

autodoc_member_order = 'bysource'
# None is read as making the option True (cf autodoc documentation)
autodoc_default_options = {
    'members': None,
    'special-members': None,
    'show-inheritance': None,
}

##inheritance_node_attrs = dict(shape='ellipse', fontsize=14, height=0.75,
##                              color='dodgerblue1', style='filled')

#: rankdir ['LR', 'TB', 'BT', 'RL']
inheritance_graph_attrs = dict(
    rankdir="LR",
    width="8.0",
    #size='"8.0, 20.0"',
    fontsize=14,
    ratio='compress'
    )

autosummary_generate = True

# Napoleon config
napoleon_include_special_with_doc = False


# -- Options for HTML output  -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
# html_theme = 'nucleic_doc'
#'default'
#'agogo'
#
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    'navigation_depth' : 3,
#    #'navbar_class': 'navbar-inverse',
#    'navbar_sidebarrel': False,
#    'bootswatch_theme': 'cosmo',
#    'bootstrap_version': '3',
}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = ['./']
#html_theme_path = sphinx_bootstrap_theme.get_html_theme_path()

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
html_short_title = project + " v" + release

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = '_static/logo-rev.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = 'nucleic.ico'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    '**': ['realglobaltoc.html'],
    'index': [],
    'genindex': [],
    'search': [],
}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_domain_indices = True

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'enamldoc'


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'enaml.tex', 'Enaml Documentation',
   'Nucleic Development Team', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
latex_use_parts = True

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Additional stuff for the LaTeX preamble.
#latex_preamble = ''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'enaml', 'Enaml Documentation',
     'Nucleic Development Team', 1)
]


# -- Options for Epub output ---------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = 'Enaml'
epub_author = 'Chris Colbert'
epub_publisher = 'Nucleic Development Team'
epub_copyright = '2013-2024, Nucleic Development Team'

# The language of the text. It defaults to the language option
# or en if the language is not set.
#epub_language = ''

# The scheme of the identifier. Typical schemes are ISBN or URL.
#epub_scheme = ''

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#epub_identifier = ''

# A unique identification for the text.
#epub_uid = ''

# HTML files that should be inserted before the pages created by sphinx.
# The format is a list of tuples containing the path and title.
#epub_pre_files = []

# HTML files shat should be inserted after the pages created by sphinx.
# The format is a list of tuples containing the path and title.
#epub_post_files = []

# A list of files that should not be packed into the epub file.
#epub_exclude_files = []

# The depth of the table of contents in toc.ncx.
#epub_tocdepth = 3

# Allow duplicate toc entries.
#epub_tocdup = True


# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    'python' : ('http://docs.python.org/', None),
    #'atom' : ('https://github.com/nucleic/atom/', None)
    }



#------------------------------------------------------------------------------
# Copyright (c) 2022-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#------------------------------------------------------------------------------
from textwrap import dedent
from utils import compile_source


def test_tracer_load_attr():
    source = dedent("""\
    from atom.api import Atom, Str
    from enaml.widgets.api import Window, Container, Label

    class Model(Atom):
        value = Str("foo")

    enamldef Main(Window):
        alias lbl
        attr model = Model()
        Container:
            Label: lbl:
                text << model.value
    """)
    Main = compile_source(source, 'Main')
    window = Main()
    assert window.lbl.text == "foo"
    window.model.value = "bar"
    assert window.lbl.text == "bar"


def test_tracer_call_func():
    source = dedent("""\
    from atom.api import Atom, Int
    from enaml.widgets.api import Window, Container, Label

    class Model(Atom):
        value = Int(1)

    enamldef Main(Window):
        alias lbl
        attr model = Model()
        Container:
            Label: lbl:
                text << str(model.value)
    """)
    Main = compile_source(source, 'Main')
    window = Main()
    assert window.lbl.text == "1"
    window.model.value = 2
    assert window.lbl.text == "2"


def test_tracer_binary_subscr():
    source = dedent("""\
    from atom.api import Atom, ContainerList
    from enaml.widgets.api import Window, Container, SpinBox

    class Model(Atom):
        items = ContainerList(default=[1])

    enamldef Main(Window):
        alias sb
        attr model = Model()
        Container:
            SpinBox: sb:
                value << model.items[0]
    """)
    Main = compile_source(source, 'Main')
    window = Main()
    window.model.items[0] = 3
    assert window.sb.value == 3


def test_inverter_load_attr():
    source = dedent("""\
    from atom.api import Atom, Int
    from enaml.widgets.api import Window, Container, Label, SpinBox

    class Model(Atom):
        value = Int(0)

    enamldef Main(Window):
        alias sb
        attr model = Model()
        Container:
            SpinBox: sb:
                value := model.value
    """)
    Main = compile_source(source, 'Main')
    window = Main()
    assert window.sb.value == 0
    window.sb.value = 2
    assert window.model.value == 2
    window.model.value = 3
    assert window.sb.value == 3


def test_inverter_call_func():
    source = dedent("""\
    from atom.api import Atom, Int
    from enaml.widgets.api import Window, Container, Label, SpinBox

    class Model(Atom):
        value = Int(0)

    enamldef Main(Window):
        alias sb
        attr model = Model()
        Container:
            SpinBox: sb:
                value := getattr(model, "value")
    """)
    Main = compile_source(source, 'Main')
    window = Main()
    assert window.sb.value == 0
    window.sb.value = 2
    assert window.model.value == 2
    window.model.value = 3
    assert window.sb.value == 3


def test_inverter_binary_subscr():
    source = dedent("""\
    from atom.api import Atom, ContainerList
    from enaml.widgets.api import Window, Container, SpinBox

    class Model(Atom):
        items = ContainerList(default=[1])

    enamldef Main(Window):
        alias sb
        attr model = Model()
        Container:
            SpinBox: sb:
                value := model.items[0]
    """)
    Main = compile_source(source, 'Main')
    window = Main()
    assert window.sb.value == 1
    window.sb.value = 2
    assert window.model.items[0] == 2
    window.model.items[0] = 3
    assert window.sb.value == 3



# ------------------------------------------------------------------------------
# Copyright (c) 2013-2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ------------------------------------------------------------------------------
from __future__ import print_function

import datetime

from atom.api import Atom, Str, Range, Bool, Value, Int, Tuple, observe
import enaml
from enaml.qt.qt_application import QtApplication

import sys, os

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


class Person(Atom):
    """A simple class representing a person object."""

    last_name = Str()

    first_name = Str()

    age = Range(low=0)

    dob = Value(datetime.date(1970, 1, 1))

    debug = Bool(False)

    @observe("age")
    def debug_print(self, change):
        """Prints out a debug message whenever the person's age changes."""
        if self.debug:
            templ = "{first} {last} is {age} years old."
            s = templ.format(
                first=self.first_name,
                last=self.last_name,
                age=self.age,
            )
            print(s)

    @observe("dob")
    def update_age(self, change):
        """Update the person's age whenever their date of birth changes"""
        # grab the current date time
        now = datetime.datetime.now()
        # estimate the person's age within one year accuracy
        age = now.year - self.dob.year
        # check to see if the current date is before their birthday and
        # subtract a year from their age if it is
        if (
            now.month == self.dob.month and now.day < self.dob.day
        ) or now.month < self.dob.month:
            age -= 1
        # set the persons age
        self.age = age


class Employer(Person):
    """An employer is a person who runs a company."""

    # The name of the company
    company_name = Str()


class Employee(Person):
    """An employee is person with a boss and a phone number."""

    # The employee's boss
    boss = Value(Employer)

    # The employee's phone number as a tuple of 3 ints
    phone = Tuple(Int())

    # This method will be called automatically by atom when the
    # employee's phone number changes
    def _observe_phone(self, val):
        print("received new phone number for %s: %s" % (self.first_name, val))


def main():
    # Create an employee with a boss
    boss_john = Employer(
        first_name="John",
        last_name="Paw",
        company_name="Packrat's Cats",
    )
    employee_mary = Employee(
        first_name="Mary",
        last_name="Sue",
        boss=boss_john,
        phone=(555, 555, 5555),
    )

    # Import our Enaml EmployeeView
    with enaml.imports():
        from employee_view import EmployeeView

    app = QtApplication()
    # Create a view and show it.
    view = EmployeeView(employee=employee_mary)
    view.show()

    app.start()


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------------------
# Copyright (c) 2022-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ------------------------------------------------------------------------------


def test_loading_dock_area_images(qapp):
    from enaml.qt.QtCore import QRect, QPoint
    from enaml.qt.docking.q_guide_rose import BorderGuide, CompassGuide, CompassExGuide

    # This forces to load all the images
    bg = BorderGuide()
    bg.layout(QRect())
    cg = CompassGuide()
    cg.layout(QPoint())
    ceg = CompassExGuide()
    ceg.layout(QPoint())



#------------------------------------------------------------------------------
# Copyright (c) 2013-2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
import sys
from setuptools import Extension, setup
from setuptools.command.build_py import build_py
from setuptools.command.develop import develop

from cppy import CppyBuildExt

# Use the env var ENAML_DISABLE_FH4 to disable linking against VCRUNTIME140_1.dll

ext_modules = [
    Extension(
        'enaml.weakmethod',
        ['enaml/src/weakmethod.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.callableref',
        ['enaml/src/callableref.cpp'],
        language='c++',
    ),
    Extension(
       'enaml.signaling',
       ['enaml/src/signaling.cpp'],
       language='c++',
    ),
    Extension(
        'enaml.core.funchelper',
        ['enaml/src/funchelper.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.colorext',
        ['enaml/src/colorext.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.fontext',
        ['enaml/src/fontext.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.core._dynamicscope',
        ['enaml/src/dynamicscope.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.core.alias',
        ['enaml/src/alias.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.core.declarative_function',
        ['enaml/src/declarative_function.cpp'],
        language='c++',
    ),
    Extension(
        'enaml.core.subscription_observer',
        ['enaml/src/subscription_observer.cpp'],
        language='c++',
    ),
]


if sys.platform == 'win32':
    ext_modules.append(
        Extension(
            'enaml.winutil',
            ['enaml/src/winutil.cpp'],
            libraries=['user32', 'gdi32'],
            language='c++'
        )
    )


setup(
    use_scm_version=True,
    ext_modules=ext_modules,
    cmdclass={'build_ext': CppyBuildExt},
)



#------------------------------------------------------------------------------
# Copyright (c) 2017-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------

MANNI_THEME = {
    "python": {
        "comment": {
            "color": "#0099FF",
            "font-style": "italic"
        },
        "single_quoted_string": {
            "color": "#CC3300"
        },
        "class_name": {
            "color": "#00AA88",
            "font-weight": "bold"
        },
        "function_method_name": {
            "color": "#CC00FF"
        },
        "operator": {
            "color": "#555555"
        },
        "double_quoted_string": {
            "color": "#CC3300"
        },
        "triple_double_quoted_string": {
            "color": "#CC3300"
        },
        "decorator": {
            "color": "#9999FF"
        },
        "comment_block": {
            "color": "#0099FF",
            "font-style": "italic"
        },
        "keyword": {
            "color": "#006699",
            "font-weight": "bold"
        },
        "triple_single_quoted_string": {
            "color": "#CC3300"
        },
        "unclosed_string": {
            "color": "#AA0000",
            "paper": "#FFAAAA"
        },
        "highlighted_identifier": {
            "paper": "#ffffcc"
        }
    },
    "settings": {
        "caret": "#000000",
        "color": "#000000",
        "paper": "#f0f3f3",
        "name": "manni"
    }
}

MANNI_THEME['enaml'] = MANNI_THEME['python']



# ------------------------------------------------------------------------------
# Copyright (c) 2025, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ------------------------------------------------------------------------------
import pytest
from atom.api import Atom, Value
from enaml.widgets.api import Label
from enaml.core.subscription_observer import SubscriptionObserver


def test_subscription_observer_ref():
    label = Label()
    observer = SubscriptionObserver(label, "text")
    assert bool(observer)
    assert observer.name == "text"
    assert observer.ref() is label
    observer.ref = None
    assert not bool(observer)


def test_subscription_observer_new():
    label = Label()
    with pytest.raises(TypeError):
        observer = SubscriptionObserver(label, 0)

    with pytest.raises(TypeError):
        observer = SubscriptionObserver(None, "")


def test_subscription_observer_update():
    class Engine(Atom):
        owner = Value()
        name = Value()

        def update(self, owner, name):
            self.owner = owner
            self.name = name

    class Owner(Atom):
        _d_engine = Value()

    engine = Engine()
    owner = Owner(_d_engine=engine)

    observer = SubscriptionObserver(owner, "text")
    observer()
    assert engine.owner is owner
    assert engine.name is "text"



# ------------------------------------------------------------------------------
# Copyright (c) 2022-2024, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ------------------------------------------------------------------------------
import asyncio
import os
import sys
from inspect import iscoroutine
from queue import Empty, Queue

import enaml
from atom.api import Bool, Typed
from enaml.qt.qt_application import QtApplication

try:
    from asyncqtpy import QEventLoopPolicy
except ImportError:
    try:
        from qasync import DefaultQEventLoopPolicy as QEventLoopPolicy
    except ImportError:
        print("Please install 'asyncqtpy' or 'qasync'")
        sys.exit(1)


sys.path.append(os.path.dirname(os.path.abspath(__file__)))


class AsyncQtApplication(QtApplication):
    queue = Typed(Queue, ())
    running = Bool()

    def start(self):
        try:
            self.running = True
            loop = asyncio.new_event_loop()
            with loop:
                loop.run_until_complete(self.main())
            super().stop()
        except RuntimeError:
            pass  # Loop stopped
        finally:
            self.running = False

    async def main(self):
        """Run any async deferred calls in the main ui loop."""
        while self.running:
            try:
                await self.queue.get(block=False)
            except Empty:
                await asyncio.sleep(0.1)
            except Exception as e:
                # Handle errors here
                raise e

    def stop(self):
        self.running = False

    def deferred_call(self, callback, *args, **kwargs):
        if iscoroutine(callback):
            return self.queue.put(callback)
        return super().deferred_call(callback, *args, **kwargs)

    def timed_call(self, ms, callback, *args, **kwargs):
        if iscoroutine(callback):
            return super().timed_call(ms, self.queue.put, callback)
        return super().timed_call(callback, *args, **kwargs)


def main():
    asyncio.set_event_loop_policy(QEventLoopPolicy())

    with enaml.imports():
        from async_view import Main

    app = AsyncQtApplication()

    view = Main()
    view.show()

    # Start the application event loop
    app.start()


if __name__ == "__main__":
    main()
